;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Heart
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          JSR SUB_GFX             ; graphics routine
                    LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /
					JSL $27FAF4 ;SUB_OFF_SCREEN_HB

                    JSR CHECK_DAMAGE
					LDA $AA,x
					STA $151C,x
					JSL $01802A             ; update position based on speed values
					LDA $151C,x
					STA $AA,x
					LDA $1528,x
					BNE NOT_MODE1
					JSR MODE1
					RTS
NOT_MODE1			CMP #$01
					BNE NOT_MODE2
					JSR MODE2
					RTS
NOT_MODE2			CMP #$02
					BNE NOT_MODE3
					JSR MODE3
					RTS
NOT_MODE3			CMP #$03
					BNE NOT_MODE4
					JSR MODE4
					RTS
NOT_MODE4			JSR MODE5
MAIN_E				RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder 
											
                    PHX                     ; push sprite index

                    LDA $00                 ; \ tile x position = sprite x location ($00)
                    STA $0300,y             ; /                    
                                        
                    LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    STA $0301,y             ; /
                                 
					LDA #$2C
                    STA $0302,y             ; / 

                    LDA #$00
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | ?? what is in 64, level properties... disable layer priority??
                    STA $0303,y             ; / store tile properties

                    PLX                     ; pull, X = sprite index
                    
                    LDY #$02
                    LDA #$00                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SPEED1				dcb $20,$E0

CHECK_DAMAGE		JSL $01A7DC             ; check for mario/sprite contact
                    BCC RETURN_24           ; (carry set = contact)
                    LDA $1497               ; \ if mario is invincible...
                    BNE RETURN_24             ; /   ... return
                    JSL $00F5B7
RETURN_24           RTS                     ; return 

MODE1				LDA $157C,x
					PHX
					TAX
					LDA SPEED1,x
					PLX
					STA $B6,x
					STZ $AA,x
					LDA $14D4,x
					BEQ MODE1_E
					STZ $14C8,x
MODE1_E				RTS
MODE2				LDA #$20
					STA $AA,x
					STZ $B6,x
					RTS

MODE3				LDA #$E8
					STA $AA,x
					STZ $B6,x
					RTS

MODE4				LDA $1570,x
					BNE MODE4_2
					LDA #$10
					STA $AA,x
					STZ $B6,x
					INC $1570,x
					RTS
MODE4_2				CMP #$01
					BNE MODE4_3
					DEC $AA,x
					LDA $AA,x
					CMP #$80
					BCS MODE4_2_2
					DEC $B6,x
					BRA MODE4_2_3
MODE4_2_2			INC $B6,x
MODE4_2_3			LDA $AA,x
					CMP #$F0
					BNE MODE4_E
					INC $1570,x
					RTS
MODE4_3				CMP #$02
					BNE MODE4_4
					INC $AA,x
					LDA $AA,x
					CMP #$80
					BCS MODE4_3_2
					DEC $B6,x
					BRA MODE4_3_3
MODE4_3_2			INC $B6,x
MODE4_3_3			LDA $AA,x
					CMP #$18
					BNE MODE4_E
					INC $1570,x
					RTS
MODE4_4				CMP #$03
					BNE MODE4_5
					DEC $AA,x
					LDA $AA,x
					CMP #$80
					BCS MODE4_4_2
					DEC $B6,x
					BRA MODE4_4_3
MODE4_4_2			INC $B6,x
MODE4_4_3			LDA $AA,x
					CMP #$E0
					BNE MODE4_E
					INC $1570,x
					RTS
MODE4_5				INC $AA,x
					LDA $AA,x
					CMP #$80
					BCS MODE4_5_2
					DEC $B6,x
					BRA MODE4_5_3
MODE4_5_2			INC $B6,x
MODE4_5_3			LDA $AA,x
					CMP #$28
					BNE MODE4_E
					INC $1570,x
					RTS
MODE4_E				RTS

MODE5				LDA $1570,x
					BNE MODE5_2
					JSR SPEED_DEC
					LDA $1594,x
                    STA $AA,x
                    LDA $1504,x
                    STA $B6,x
					INC $1570,x
MODE5_0				LDA $1594,x
					CMP #$80
					BCS MODE5_4
					CMP #$20
					BCC MODE5_E
					LSR A
					STA $1594,x
MODE5_3_1			LDA $1504,x
					CMP #$80
					BCS MODE5_3_2
					LSR A
					STA $1504,x
					BRA MODE5_0
					RTS
MODE5_3_2			LDA #$00
					SEC
					SBC $1504,x
					LSR A
					STA $1504,x
					LDA #$00
					SEC
					SBC $1504,x
					STA $1504,x
					BRA MODE5_0
MODE5_E				RTS
MODE5_4				CMP #$E0
					BCS MODE5_E
					LDA #$E0
					STA $1594,x
					BRA MODE5_3_1
					RTS
MODE5_2				LDA $1594,x
                    STA $AA,x
                    LDA $1504,x
                    STA $B6,x
					LDA $14D4,x
					BNE MODE5_3
					LDA $14E0,x
					BNE MODE5_3
					RTS
MODE5_3				STZ $14C8,x
					RTS



SPEED_DEC           LDA $E4,x
                    SEC
                    SBC $94
                    STA $0F
                    LDA $14E0,x
                    SBC $95
                    BPL FACE_LEFT
                    JSR RIGHT_SPEED
                    BRA VERC
FACE_LEFT           CMP #$01
                    BEQ LEFT2
                    JSR LEFT_SPEED1
                    BRA VERC
LEFT2               JSR LEFT_SPEED2
VERC                LDA $D8,x
                    SEC
                    SBC $96
                    STA $0F
                    LDA $14D4,x
                    SBC $97
                    BPL FACE_UP
                    JSR DOWN_SPEED
                    LDA $1594,x
                    CMP #$10
                    BCS VER_E
                    LDA #$10
                    STA $1594,x
                    BRA VER_E
FACE_UP             JSR UP_SPEED
                    LDA $1594,x
                    CMP #$F0
                    BCC VER_E
                    LDA #$F0
                    STA $1594,x
VER_E               RTS

UP_SPEED            LDA $0F38
                    CMP #$00
                    BNE UP3
                    LDA $0F
                    CMP #$80
                    BCC UP2
                    LDA #$7F
UP2                 STA $0F
                    BRA UP5
UP3                 CMP #$02
                    BNE UP4
                    JSR UP1_1
                    BRA UP5
UP4                 JSR UP1_2
UP5                 LDA #$00
                    SEC
                    SBC $0F
                    STA $1594,x
                    RTS

DOWN_SPEED          LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    LDA $0F38
                    CMP #$00
                    BNE DOWN3
                    LDA $0F
                    CMP #$80
                    BCC DOWN2
                    LDA #$7F
DOWN2               STA $1594,x
                    RTS
DOWN3               CMP #$02
                    BNE DOWN4
                    JSR UP1_1
                    BRA DOWN5
DOWN4               JSR UP1_2
DOWN5               LDA $0F
                    STA $1594,x
                    RTS
UP1_1
LOOPUP1             LDA $0F37
                    BEQ LOOPUP1E
                    LDA $0F
                    LSR A
                    STA $0F
                    DEC $0F37
                    LDA $0F37
                    BNE LOOPUP1
LOOPUP1E            LDA $0F
                    CMP #$80
                    BCC LOOPUP1EE
                    LDA #$7F
                    STA $0F
LOOPUP1EE           RTS

UP1_2               LDA $0F
                    CMP #$80
                    BCC LOOPUP3
                    LDA #$7F
                    STA $0F
                    STZ $0F37
                    RTS
LOOPUP3             LDA $0F37
                    BEQ LOOPUP2E
                    LDA $0F
                    ASL A
                    STA $0F
                    DEC $0F37
                    LDA $0F37
                    BNE UP1_2
LOOPUP2E            LDA $0F
                    CMP #$80
                    BCC LOOPUP2EE
                    LDA #$7F
                    STA $0F
LOOPUP2EE           RTS







LEFT_SPEED1         STZ $0F38
                    LDA $0F
                    CMP #$18
                    BCC LEFT1_2
                    LDA #$02
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_40
                    BRA LEFT1_3
LEFT1_2             CMP #$0C
                    BCS LEFT1_3
                    LDA #$01
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_20
                    BRA LEFT1_3
LEFT1_3             LDA #$00
                    SEC
                    SBC $0F
                    STA $1504,x
                    RTS

LEFT1_40            
LOOP140             LDA $0F
                    CMP #$18
                    BCC LOOP140E
                    LSR A
                    STA $0F
                    INC $0F37
                    BRA LOOP140
LOOP140E            RTS
LEFT1_20            
LOOP120             LDA $0F
                    CMP #$0C
                    BCS LOOP120E
                    ASL A
                    STA $0F
                    INC $0F37
                    BRA LOOP140
LOOP120E            RTS

LEFT_SPEED2         STZ $0F37
                    INC $0F37
                    LDA $0F
                    LSR A
                    CLC
                    ADC #$80
                    STA $0F
                    LDA #$02
                    STA $0F38
                    JSR LEFT1_40
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $1504,x
                    RTS

RIGHT_SPEED         STZ $0F38
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    CMP #$18
                    BCC RIGHT_2
                    LDA #$02
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_40
                    BRA RIGHT_3
RIGHT_2             CMP #$0C
                    BCS RIGHT_3
                    LDA #$01
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_20
                    BRA RIGHT_3
RIGHT_3             LDA $0F
                    STA $1504,x
                    RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /




