;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Larry
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    EXTRA_BITS = $7FAB10
                    NEW_SPRITE_NUM = $7FAB9E    ;08 bytes   custom sprite number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
					STZ $C2,x
					STZ $1528,x
					STZ $1570,x
					STZ $1504,x
					JSR HDMA1
					STZ $14AF
					STZ $157C,x
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          LDA #$17
					STA $00212C
					LDA #$00
					STA $00212D
					LDA $C2,x
					BEQ MAIN
					JSR SUB_GFX             ; graphics routine
MAIN				LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /
					JSL $01802A             ; update position based on speed values
					LDA $157C,x
					BEQ MAIN1
					JSR WIND
MAIN1				LDA $1528,x
					BNE NOT_MODE0
					JSR MODE0
					RTS
NOT_MODE0			CMP #$01
					BNE NOT_MODE1
					JSR MODE1
					RTS
NOT_MODE1			CMP #$02
					BNE NOT_MODE2
					JSR MODE2
					RTS
NOT_MODE2			CMP #$03
					BNE NOT_MODE3
					JSR MODE3
					RTS
NOT_MODE3			JSR MODE4
					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $1C,$24,$04,$0C,$20,$00
HORZ_DISP           dcb $08,$04,$00
VERT_DISP           dcb $F8,$08,$00
TILE_SIZE            dcb $00,$02,$02

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder                
					PHX                     ; push sprite index
                    LDX #$02                ; loop counter = (number of tiles per frame) - 1
LOOP_START          PHX                     ; push current tile number
                    
                    
                    LDA $00                 ; \ tile y position = sprite y location ($01) + tile displacement
					CLC
					ADC HORZ_DISP,x
                    STA $0300,y             ; /   set y position of the tile
					
					LDA $01                ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /   set y position of the tile
                    
					LDA TILEMAP,x           ; \ store tile
                    STA $0302,y            ; / 
					
					LDA #$00            ;  |
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
					;ORA $64
                    STA $0303,y             ; / store tile properties
					PLX
					
					LDA TILE_SIZE,x
                    PHX
                    PHA
                    TYA                     ; \ get index to sprite property map ($460)...
                    LSR A                   ; |    ...we use the sprite OAM index...
                    LSR A                   ; |    ...and divide by 4 because a 16x16 tile is 4 8x8 tiles
                    TAX                     ; | 
                    PLA
                    STA $0460,x             ; /  
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

                    PLX                     ; pull, X = sprite index
                    LDY #$FF                ; \ 02, because we didn't write to 460 yet   460 = Tile Size 02:16x16
                    LDA #$02                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ETC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
WIND				LDA $13
					AND #$01
					BNE WIND_E
					LDA $13
					AND #$20
					BNE WIND_2
					LDA #$17
					STA $1DFC
WIND_2				LDA $94
					BEQ WIND_E
					DEC $94
WIND_E				RTS



RETURN0				RTS
MODE0				STZ $AA,x
					LDA $1468
					BEQ MODE0_1
					LDA $13
					AND #$01
					BNE RETURN0
					DEC $1468
					RTS
MODE0_1				LDA #$01
					STA $C2,x
					LDA $14D4,x
					BNE MODE0_2
					LDA #$FF
					STA $15AC,x
					LDA #$01
					STA $1528,x
					LDA $1570,x
					CMP #$02
					BNE RETURN0
					LDA #$01
					STA $157C,x
					RTS
MODE0_2				LDA #$F8
					STA $AA,x
					RTS

MODE1				JSR CHECK_DAMAGE
					STZ $AA,x
					JSR SHAKE
					LDA $15AC,x
					CMP #$A0
					BEQ MODE1_GEN1
					CMP #$40
					BEQ MODE1_GEN2
					CMP #$00
					BEQ MODE1_3
MODE1_2				RTS
MODE1_GEN1			LDA #$17
					STA $1DFC
					JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$20
					STA $151C,y
					;JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$10
					STA $151C,y
					JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$00
					STA $151C,y
					RTS
MODE1_GEN2			LDA #$17
					STA $1DFC
					JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$28
					STA $151C,y
					JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$18
					STA $151C,y
					JSR FIRE_GEN
					LDA #$01
					STA $1528,y
					LDA #$08
					STA $151C,y
					RTS
MODE1_3				LDA #$80
					STA $15AC,x
					LDA #$02
					STA $1528,x
					RTS
					
MODE2				STZ $AA,x
					LDA $15AC,x
					BEQ MODE2_2
					JSR CHECK_DAMAGE
					LDA $15AC,x
					CMP #$7F
					BEQ MODE2_1
					RTS
MODE2_1				JSR FIRE_GEN
					LDA #$00
					STA $1528,y
					STA $1570,y
					RTS
MODE2_2				LDA $1468
					CMP #$01
					BEQ MODE2_H
					CMP #$C0
					BEQ MODE2_3
					LDA $1468
					CMP #$20
					BNE MODE2_2_2
					STZ $C2,x
					STZ $1504,x
MODE2_2_2			INC $1468
					RTS
MODE2_3				LDA #$03
					STA $1528,x
					LDA #$FF
					STA $15AC,x
					LDA #$20
					STA $D8,x
					LDA #$01
					STA $14D4,x
					RTS
MODE2_H				LDA $1570,x
					BEQ MODE2_2_2
					CMP #$01
					BNE MODE2_H_2
					JSR HDMA2
					BRA MODE2_2_2
MODE2_H_2			JSR HDMA3
					BRA MODE2_2_2
					
MODE3				STZ $AA,x
					LDA $15AC,x
					CMP #$E0
					BEQ MODE3_1
					CMP #$40
					BEQ MODE3_2
					CMP #$00
					BEQ MODE3_3
					RTS
MODE3_1				LDA #$20
					STA $1602,x
					JSR GEN_SPRITE
					RTS
MODE3_2				LDA #$60
					STA $1602,x
					JSR GEN_SPRITE
					RTS
MODE3_3				STZ $1528,x
					RTS
MODE4				LDA #$01
					STA $C2,x
					LDA #$18
					STA $AA,x
					LDA $14D4,x
					BEQ MODE4_E
					LDA $D8,x
					CMP #$A0
					BCC MODE4_E
					LDA #$0B
                    STA $1DFB
                    LDA #$FF
                    STA $13C6
                    STA $1493
					STZ $14C8,x
MODE4_E				RTS
					
					
SHAKE				LDA $13
					AND #$01
					BEQ SHAKE_0
SHAKE_E				RTS
SHAKE_0				LDA $15AC,x
					AND #$10
					BEQ SHAKE_1
					LDA $1468
					CLC
					ADC #$01
					STA $1468
					LDA $1469
					ADC #$00
					STA $1469
					LDA $D8,x
					SEC
					SBC #$01
					STA $D8,x
					LDA $14D4,x
					SBC #$00
					STA $14D4,x
					RTS
SHAKE_1				LDA $1468
					BEQ SHAKE_E
					LDA $1468
					SEC
					SBC #$01
					STA $1468
					LDA $1469
					SBC #$00
					STA $1469
					LDA $D8,x
					CLC
					ADC #$01
					STA $D8,x
					LDA $D8,x
					ADC #$00
					STA $D8,x
					RTS

CHECK_DAMAGE		LDA $C2,x
					BEQ NO_DAMAGE
					LDA $1504,x
					BNE NO_DAMAGE2
					JSL $01A7DC
					BCC NO_DAMAGE
					LDA $7D
					CMP #$10
					BMI ENE_WINS
					INC $1570,x
					INC $1504,x
					JSL $01AA33             ; set mario speed
					JSL $01AB99             ; display contact graphic
					LDA #$20
                    STA $1DF9
					LDA $1570,x
					CMP #$03
					BEQ THEEND
NO_DAMAGE			RTS
NO_DAMAGE2			LDA $1504,x
					CMP #$30
					BEQ NO_DAMAGE3
					INC $1504,x
					LDA $D8,x
					CLC
					ADC #$01
					STA $D8,x
					LDA $14D4,x
					ADC #$00
					STA $14D4,x
					RTS
NO_DAMAGE3			STZ $C2,x
					RTS
					

ENE_WINS            LDA $1497               ; \ if mario is invincible...
                    BNE RETURN2             ; /   ... return
                    JSL $00F5B7             ; hurt mario
RETURN2             RTS
THEEND				LDA #$04
					STA $1528,x
					LDA #$2B
					STA $1DFC
					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; generate sprite1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURNK68           RTS     
GEN_SPRITE          JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURNK68            ; / after: Y has index of sprite being generated

                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /

                    LDA #$2B
                    STA $009E,y
                    
                    LDA $1602,x
                    STA $00E4,y

                    LDA #$00
                    STA $14E0,y             ; /

                    LDA #$B0
					STA $00D8,y             ;  |
                    LDA #$00
					STA $14D4,y             ; /

                    PHX
                    TYX
                    JSL $07F7D2             ;  
                    PLX

                    RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite generator routine2
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


RETURN67            RTS
FIRE_GEN            JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURN67            ; / after: Y has index of sprite being generated
                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /
                    PHX
                    LDA NEW_SPRITE_NUM,x
					INC A
					TYX
                    STA NEW_SPRITE_NUM,x
                    PLX
                    
                    LDA #$98
                    STA $00E4,y

                    LDA #$00
                    STA $14E0,y             ; /

                    LDA #$30
                    STA $00D8,y             ;  |
                    LDA #$01
                    STA $14D4,y             ; /
                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    JSL $0187A7             ;  get table values for custom sprite
                    LDA #$88
                    STA EXTRA_BITS,x
                    PLX
                    RTS                          ; return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; HDMA1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
HDMA1				LDA $7FFB28
					BNE HDMA1_LBL1
					LDA #$20
					STA $7FFB28
					LDA #$40
					STA $7FFB30
					
					LDA #$00
					STA $7FFB29
					LDA #$02
					STA $7FFB31
					
					LDA #$21
					STA $7FFB2A
					LDA #$22
					STA $7FFB32
					
					LDA #$00
					STA $7FFB2B
					STA $7FFB33
					LDA #$E1
					STA $7FFB2C
					LDA #$E2
					STA $7FFB34
					LDA #$7F
					STA $7FFB2D
					STA $7FFB35
			
HDMA1_LBL1			LDA #$50
					STA $7FE100
					STA $7FE200
					LDA #$00
					STA $7FE101
					LDA #$22
					STA $7FE201
					LDA #$18
					STA $7FE202
					
					LDA #$0A
					STA $7FE102
					STA $7FE203
					LDA #$00
					STA $7FE103
					LDA #$21
					STA $7FE204
					LDA #$28
					STA $7FE205
					
					LDA #$0A
					STA $7FE104
					STA $7FE206
					LDA #$00
					STA $7FE105
					LDA #$01
					STA $7FE207
					LDA #$3C
					STA $7FE208
					
					LDA #$0A
					STA $7FE106
					STA $7FE209
					LDA #$00
					STA $7FE107
					LDA #$41
					STA $7FE20A
					LDA #$54
					STA $7FE20B
					
					LDA #$0A
					STA $7FE108
					STA $7FE20C
					LDA #$00
					STA $7FE109
					LDA #$81
					STA $7FE20D
					LDA #$68
					STA $7FE20E
					
					LDA #$0A
					STA $7FE10A
					STA $7FE20F
					LDA #$00
					STA $7FE10B
					LDA #$A1
					STA $7FE210
					LDA #$74
					STA $7FE211
					
					LDA #$0A
					STA $7FE10C
					STA $7FE212
					LDA #$00
					STA $7FE10D
					LDA #$46
					STA $7FE213
					LDA #$7D
					STA $7FE214
					
					LDA #$0A
					STA $7FE10E
					STA $7FE215
					LDA #$00
					STA $7FE10F
					LDA #$A9
					STA $7FE216
					LDA #$7D
					STA $7FE217
					
					LDA #$0A
					STA $7FE110
					STA $7FE218
					LDA #$00
					STA $7FE111
					LDA #$0D
					STA $7FE219
					LDA #$7E
					STA $7FE21A
					
					LDA #$0A
					STA $7FE112
					STA $7FE21B
					LDA #$00
					STA $7FE113
					LDA #$71
					STA $7FE21C
					LDA #$7E
					STA $7FE21D
					
					LDA #$00
					STA $7FE114
					STA $7FE21E
					RTS
					
HDMA2				LDA #$26
					STA $7FE201
					LDA #$18
					STA $7FE202
					
					LDA #$28
					STA $7FE204
					LDA #$28
					STA $7FE205
					
					LDA #$0F
					STA $7FE207
					LDA #$38
					STA $7FE208

					LDA #$35
					STA $7FE20A
					LDA #$54
					STA $7FE20B

					LDA #$3A
					STA $7FE20D
					LDA #$68
					STA $7FE20E

					LDA #$3D
					STA $7FE210
					LDA #$58
					STA $7FE211

					LDA #$3F
					STA $7FE213
					LDA #$48
					STA $7FE214

					LDA #$3F
					STA $7FE216
					LDA #$3D
					STA $7FE217

					LDA #$DF
					STA $7FE219
					LDA #$35
					STA $7FE21A

					LDA #$1F
					STA $7FE21C
					LDA #$1E
					STA $7FE21D

					RTS
					
HDMA3				LDA #$01
					STA $14AF
					LDA #$22
					STA $7FE201
					LDA #$18
					STA $7FE202
					
					LDA #$23
					STA $7FE204
					LDA #$20
					STA $7FE205
					
					LDA #$84
					STA $7FE207
					LDA #$24
					STA $7FE208

					LDA #$86
					STA $7FE20A
					LDA #$2C
					STA $7FE20B

					LDA #$A7
					STA $7FE20D
					LDA #$38
					STA $7FE20E

					LDA #$A5
					STA $7FE210
					LDA #$34
					STA $7FE211

					LDA #$84
					STA $7FE213
					LDA #$2C
					STA $7FE214

					LDA #$A5
					STA $7FE216
					LDA #$38
					STA $7FE217

					LDA #$C6
					STA $7FE219
					LDA #$40
					STA $7FE21A

					LDA #$E7
					STA $7FE21C
					LDA #$4C
					STA $7FE21D
					
					RTS
