;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Lift in Ruin 
;;   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    ;symbolic names for ram addresses
                    MARIO_STATUS    = $19 
                    MARIO_X_SPEED   = $7B
                    MARIO_Y_SPEED   = $7D
                    MARIO_X_POS     = $94
                    MARIO_X_POS_HI  = $95
                    MARIO_Y_POS     = $96 
                    MARIO_Y_POS_HI  = $97
                    SPRITE_Y_SPEED  = $AA
                    SPRITE_X_SPEED  = $B6
                    SPRITE_STATE    = $C2
                    SPRITE_Y_POS    = $D8
                    SPRITE_X_POS    = $E4
                    SPRITE_Y_POS_HI = $14D4
                    SPRITE_X_POS_HI = $14E0
                    SPR_OBJ_STATUS  = $1588
                    IS_ON_YOSHI     = $187A 
                    SPRITE_OAM      = $15EA
                    
                    EXTRA_BITS = $7FAB10
                    
                    ON_THIS         = $1528
                    
                    ;data
                    YOSHI_OFFSET    = $10
                    SIZE            = $08
                    
                    Y_SPEED         = $10
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
                    STZ ON_THIS,x
					STZ $1570,x
INIE
                    RTL
      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"                                    
                    PHB                     
                    PHK                     
                    PLB                     
                    JSR SPRITE_CODE_START   
                    PLB                     
                    RTL      


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPRITE_CODE_START   JSL $27FAF4 ;SUB_OFF_SCREEN_X0
					JSR SUB_GFX
					
					LDA $9D
                    BNE BAIL
                    JSR SUB_B852
                    JSR SUB_MOVEMENT
BAIL                RTS                     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_MOVEMENT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_MOVEMENT                          
                    LDA #$00
                    STA SPRITE_X_SPEED,x
					LDA #$00
                    STA SPRITE_Y_SPEED,x
					
					LDA $14AF
					BNE DONT_MOVE_MARIO
					LDA $1570,x
					BEQ DONT_MOVE_MARIO
					LDA $77
					CMP #$08
					BCS DONT_MOVE_MARIO
					
					LDA EXTRA_BITS,x
                    AND #$04
					BNE DOWN
					LDA #$F0
                    STA SPRITE_Y_SPEED,x
					BRA NOT_MOVE
DOWN				LDA #$10
					STA SPRITE_Y_SPEED,x
NOT_MOVE
					LDA ON_THIS,x
                    BEQ DONT_MOVE_MARIO

MOVE_MARIO          LDA EXTRA_BITS,x
                    AND #$04
					BEQ MOVE_M2	
					
					LDA MARIO_Y_POS
					CLC
					ADC #$01
					STA MARIO_Y_POS
					LDA MARIO_Y_POS_HI
					ADC #$00
					STA MARIO_Y_POS_HI
					BRA DONT_MOVE_MARIO
					
MOVE_M2				LDA MARIO_Y_POS
					SEC
					SBC #$01
					STA MARIO_Y_POS
					LDA MARIO_Y_POS_HI
					SBC #$00
					STA MARIO_Y_POS_HI
					BRA DONT_MOVE_MARIO
					
DONT_MOVE_MARIO                    
                    
                    JSL $018022             ;update x position, no gravity
                    JSL $01801A             ;update y position, no gravity ;#
                    RTS


                    
                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $A0,$A2,$A4,$A6
X_OFFSET            dcb $00,$10,$20,$30

SUB_GFX 			JSR GET_DRAW_INFO
					

                    PHX                     ; push sprite index

                    LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START          LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
					STA $0301,y             ; /
                    
                    LDA TILEMAP,x
                    STA $0302,y             ; / 
					
					LDA $00                 ; \ tile x position = sprite x location ($00)
                    CLC
                    ADC X_OFFSET,x
                    STA $0300,y             ; /
        			
					PHX
                    LDA #$00
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | ?? what is in 64, level properties... disable layer priority??
                    STA $0303,y             ; / store tile properties
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

                    PLX                     ; pull, X = sprite index
                                              
                    
                    STZ $00       
                    STZ $01       
                    STZ $02       
                    STZ $03       
                    LDY #$00                 
                    LDA #SIZE   
                    STA $0000,y   
                    LSR A         
                    STA $0001,y   
                                        
                    LDA $00       
                    PHA           
                    LDA $02       
                    PHA       
                    
                    LDY #$02                ; \ 02, because we didn't write to 460 yet
                    LDA #$03                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen

                    PLA           
                    STA $02       
                    PLA           
                    STA $00       
                    RTS                           


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; platform code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_B852            STZ ON_THIS,x
					LDA $15C4,x 
                    ORA $154C,x   
                    BNE RETURN05
                          
                    LDA $71       ;Action (5/6=Trigger screen exit)
                    CMP #$01                
                    BCS RETURN05     
                    
                    JSR SUB_B8FF
                    BCC RETURN05      
                    
                    LDA SPRITE_Y_POS,x    
                    SEC          
                    SBC $1C      
                    STA $02      
                    SEC          
                    SBC $0D      
                    STA $09      
                    LDA $80      
                    CLC          
                    ADC #$18     
                    CMP $09      
                    BCS LBL06      
                    LDA MARIO_Y_SPEED    
                    BMI RETURN05     
                    STZ MARIO_Y_SPEED  
                        
                    LDA #$01     
                    STA $1471    ;01 is for the floating rock
                    LDA $0D      
                    CLC          
                    ADC #$1C     
                    STA $00      
                    
                    LDA #$01
                    STA ON_THIS,x
					STA $1570,x
                    

                    LDA SPRITE_Y_POS,x    
                    SEC          
                    SBC $00      
                    STA MARIO_Y_POS      
                    LDA SPRITE_Y_POS_HI,x  
                    SBC #$00     
                    STA MARIO_Y_POS_HI      
                    LDY #$00     
                    LDA $1491               ;amount to move mario 
                    BPL LBL04      
                    DEY
LBL04
                    CLC                     
                    ADC MARIO_X_POS       
                    STA MARIO_X_POS       
                    TYA           
                    ADC MARIO_X_POS_HI       
                    STA MARIO_X_POS_HI       
RETURN05            RTS           

LBL06               LDA $02       
                    CLC           
                    ADC $0D       
                    STA $02       
                    LDA #$FF      
                    LDY $73       
                    BNE LBL07      
                    LDY MARIO_STATUS       
                    BNE SMALL_MARIO
LBL07
                    LDA #$08   
SMALL_MARIO
                    CLC                     
                    ADC $80      
                    CMP $02      
                    BCC LBL10      
                    LDA MARIO_Y_SPEED       
                    BPL LBL09    
                    LDA #$10                
                    ;STA MARIO_Y_SPEED    
LBL09
                    RTS       
LBL10
                    LDA $0E       
                    CLC           
                    ADC #$10      
                    STA $00       
                    LDY #$00      
                    LDA SPRITE_X_POS,x     
                    SEC           
                    SBC $1A       
                    CMP $7E       
                    BCC LBL11     
                    LDA $00       
                    EOR #$FF      
                    INC A         
                    STA $00       
                    DEY     
LBL11
                    LDA SPRITE_X_POS,x     
                    CLC           
                    ADC $00       
                    ;STA MARIO_X_POS       
                    TYA           
                    ADC SPRITE_X_POS_HI,x   
                    ;STA MARIO_X_POS_HI       
                    ;STZ MARIO_X_SPEED       
                    RTS                     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; make standable
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_B8FF
                    LDA $00      
                    STA $0E      
                    LDA $02      
                    STA $0D      
                    LDA SPRITE_X_POS,x    
                    CLC               
					ADC #$06     ;#Jnʒu
                    STA $04      
                    LDA SPRITE_X_POS_HI,x  
                    ADC #$00     
                    STA $0A      
                    LDA $00      
                    ASL A        
                    CLC          
                    ADC #$24     ;#TCY
                    STA $06      

                    LDA SPRITE_Y_POS,x    
                    SEC          
                    SBC $02
                    STA $05      
                    LDA SPRITE_Y_POS_HI,x  
                    SBC #$00     
                    STA $0B      
                    LDA $02      
                    ASL A        
                    CLC          
                    ADC #$20    
                    STA $07      
                    JSL $03B664  
                    JSL $03B72B  
                    RTS 

                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA SPRITE_X_POS_HI,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA SPRITE_X_POS_HI,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA SPRITE_Y_POS_HI,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY SPRITE_OAM,x        ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /
                    



