;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Morton
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
					STZ $157C,x
					STZ $151C,x
					STZ $1570,x
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          JSR SUB_GFX             ; graphics routine
					LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /

					JSL $01802A             ; update position based on speed values
					
					LDA $7FFFA1
					STA $E4,x
					LDA $7FFFA2
					STA $14E0,x
					JSL $01A7DC             ; check for mario/hammer bro contact 
					JSR MODE_CHECK
					LDA $151C,x
					BNE NOT_MODE0
					JSR MODE0
					RTS
 NOT_MODE0			CMP #$01
 					BNE NOT_MODE1
					JSR MODE1
 					RTS
 NOT_MODE1			CMP #$02
 					BNE NOT_MODE2
					JSR MODE2
 					RTS
 NOT_MODE2			CMP #$03
 					BNE NOT_MODE3
					JSR MODE3
 					RTS
 NOT_MODE3			JSR MODE4
 					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $06,$08,$26,$28
					dcb $00,$02,$6C,$6E
					dcb $02,$00,$6E,$6C
					dcb $0A,$0C,$2A,$2C
					dcb $44,$46,$68,$6A
					dcb $44,$46,$64,$66
					dcb $80,$82,$A0,$A2
HORZ_DISP           dcb $00,$10,$00,$10
					dcb $F8,$08,$F8,$08
					dcb $F8,$08,$F8,$08
					dcb $F8,$08,$F8,$08
					dcb $00,$10,$00,$10
					dcb $00,$10,$00,$10
					dcb $F8,$08,$F8,$08
VERT_DISP           dcb $F6,$F6,$06,$06
PROPERTIES          dcb $00,$40

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder                 
					PHX                     ; push sprite index
					LDA $1602,x
					ASL A
					ASL A
					STA $03
					LDA $157C,x
					STA $02

                    LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START          PHX                     ; push current tile number
                    
                    
					
					LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /   set y position of the tile
                    
					PHX
					TXA
					CLC
					ADC $03
					TAX
                    LDA TILEMAP,x           ; \ store tile
                    STA $0302,y             ; / 
					LDA $00
					CLC
					ADC HORZ_DISP,x
                    STA $0300,y
					PLX
					

					PHX
					LDA $02
					TAX
					LDA PROPERTIES,x
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | put in level properties
                    STA $0303,y             ; / store tile properties
					PLX
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

                    PLX                     ; pull, X = sprite index
                    LDY #$02                ; \ 02, because we didn't write to 460 yet   460 = Tile Size 02:16x16
                    LDA #$03                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TIME_TO_LOCK = $30

MODE_CHECK			LDA $151C,x
					CMP #$04
					BNE CHECK1
					RTS
CHECK1				BEQ CHECK2
					LDA #$03
					STA $151C,x
					LDA $1570,x
					BEQ CHECK2
					LDA #$03
					STA $151C,x
					RTS
CHECK2				LDA $7FFFA4
					BEQ NOT_MODE_2
					LDA #$02
					STA $151C,x
					RTS
NOT_MODE_2			LDA $7FFFA3
					CMP #$90
					BCS MODE_1
					STZ $151C,x
					RTS
MODE_1				LDA #$01
					STA $151C,x
					RTS

MODE0				LDA #$10
					STA $AA,x
					RTS
					
					
MODE1				STZ $1602,x
					LDA #$10
					STA $AA,x
					STZ $157C,x
					LDA $15AC,x
					BNE MODE1_1
					LDA #$A3
					STA $15AC,x
					RTS
MODE1_1				CMP #$40
					BCS MODE1_E
					CMP #$38
					BCC MODE1_2
					LDA #$01
					STA $157C,x
					LDA #$02
					STA $1602,x ;38-40
					RTS
MODE1_2				CMP #$30
					BCC MODE1_3
					LDA #$01
					STA $1602,x ;30-38
					RTS
MODE1_3				CMP #$2F
					BNE MODE1_4
					LDA $94
					ADC $13
					AND #$0F
					LSR A
					LSR A
					STA $1504,x
					LDA #$01
					STA $1602,x ;2F
					JSR GEN_SPRITE
					RTS
MODE1_4				CMP #$10
					BCC MODE1_5
					LDA $1504,x
					CMP #$02
					BCS MODE1_4_2
					LDA #$03
					STA $1602,x ;10-2F
					RTS
MODE1_4_2			LDA #$01
					STA $1602,x ;10-2F
					RTS
MODE1_5				CMP #$08
					BCC MODE1_6
					LDA #$01
					STA $1602,x ;08-10
					RTS
MODE1_6				LDA #$01
					STA $157C,x
					LDA #$02
					STA $1602,x ;00-08
MODE1_E				RTS
					
MODE2				LDA $1463
					CMP #$13
					BCC MODE2_0
					LDA #$04
					STA $151C,x
					LDA #$FF
					STA $15AC,x
					RTS
MODE2_0				LDA $7FFFA5
					CMP #$18
					BNE MODE2_E
					STZ $1602,x
					LDA #$10
					STA $AA,x
					STZ $157C,x
					LDA $15AC,x
					BNE MODE2_1
					LDA #$A3
					STA $15AC,x
					RTS
MODE2_1				CMP #$40
					BCS MODE2_E
					CMP #$38
					BCC MODE2_2
					LDA #$01
					STA $157C,x
					LDA #$02
					STA $1602,x ;38-40
					RTS
MODE2_2				CMP #$30
					BCC MODE2_3
					LDA #$01
					STA $1602,x ;30-38
					RTS
MODE2_3				CMP #$2F
					BNE MODE2_4
					LDA $94
					ADC $13
					AND #$0F
					LSR A
					LSR A
					STA $1504,x
					CMP #$02
					BNE MODE2_3_2
					LDA #$01
					STA $1570,x
					RTS
MODE2_3_2			LDA #$01
					STA $1602,x ;2F
					JSR GEN_SPRITE
					RTS
MODE2_4				CMP #$10
					BCC MODE2_5
					LDA $1504,x
					CMP #$02
					BCS MODE2_4_2
					LDA #$03
					STA $1602,x ;10-2F
					RTS
MODE2_4_2			LDA #$01
					STA $1602,x ;10-2F
					RTS
MODE2_5				CMP #$08
					BCC MODE2_6
					LDA #$01
					STA $1602,x ;08-10
					RTS
MODE2_6				LDA #$01
					STA $157C,x
					LDA #$02
					STA $1602,x ;00-08
MODE2_E				RTS


MODE3				LDA #$04
					STA $1602,x
					LDA $15AC,x
					CMP #$A5
					BNE MODE3_1
					STZ $1570,x
MODE3_1				CMP #$10
					BCS MODE3_E
					CMP #$0F
					BNE MODE3_2
					LDA #$A0
					STA $AA,x
					RTS
MODE3_2				LDA $1588,x
					AND #$04
					BEQ IN_AIR
					LDA #$C0
					STA $15AC,x
					LDA #TIME_TO_LOCK
					STA $1887
					LDA #$09
                    STA $1DFC
					LDA $77
                    AND #$04
                    BEQ MODE3_E
                    LDA #TIME_TO_LOCK       ; set timer to freeze mario
                    STA $18BD
					RTS
IN_AIR				LDA #$05
					STA $1602,x
MODE3_E				RTS


MODE4				STZ $1602,x
					STZ $157C,x
					LDA $15AC,x
					CMP #$F0
					BCS MODE4_E
					CMP #$E8
					BCC MODE4_1
					LDA #$01
					STA $157C,x
					LDA #$02
					STA $1602,x
					RTS
MODE4_1				CMP #$E0
					BCC MODE4_2
					LDA #$01
					STA $1602,x
					RTS
MODE4_2				CMP #$DF
					BNE MODE4_3
					LDA #$03
					STA $1504,x
					JSR GEN_SPRITE
					LDA #$01
					STA $1602,x
					RTS
MODE4_3				PHX
					LDA $7FFFA7
					TAX
					LDA $7FFFA1
					STA $E4,x
					LDA $7FFFA2
					STA $14E0,x
					STZ $AA,x
					PLX
					LDA $15AC,x
					CMP #$A0
					BCC MODE4_4
					LDA #$01
					STA $1602,x
					RTS
MODE4_4				LDA #$06
					STA $1602,x
					LDA $15AC,x
					CMP #$30
					BNE MODE4_E
					LDA #$0B
                    STA $1DFB
                    LDA #$FF
                    STA $13C6
                    STA $1493
MODE4_E				RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPRITE_NUM			dcb $B3,$B3,$1B,$0D
STATUS_NUM			dcb $08,$08,$08,$09
TIME_TO_EXPLODE     dcb $00,$00,$04,$40

RETURN68            RTS     
GEN_SPRITE		    JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURN68            ; / after: Y has index of sprite being generated


                    PHX
					LDA $1504,x
					TAX
					LDA SPRITE_NUM,x
                    STA $9E,y
					LDA STATUS_NUM,x
					STA $14C8,y
					PLX

                    LDA $E4,x               
                    SEC
					SBC #$08
                    STA $00E4,y
    
                    LDA $14E0,x             
					SBC #$00
                    STA $14E0,y      

                    LDA $D8,x               ; \ set y position for new sprite
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    STA $14D4,y             ; /

                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    PLX                     ; / 

                    LDA #$F0
                    STA $B6,y
					LDA #$01
					STA $157C,y
                    
                    PHX
					LDA $1504,x
					TAX
					LDA TIME_TO_EXPLODE,x
					PLX
                    STA $1540,y
					TYA
					STA $7FFFA7         
					LDA $1504,x
					CMP #$02
					BCS RETURN67
					LDA #$17
					STA $1DFC
                    
RETURN67            RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /



