;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SUPU
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    EXTRA_BITS = $7FAB10
                    NEW_SPRITE_NUM = $7FAB9E    ;08 bytes   custom sprite number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
					LDA #$F1
					STA $15AC,x
					LDA #$01
					STA $157C,x
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          JSR SUB_GFX             ; graphics routine
                    LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /

                    JSL $01A7DC             ; check for mario/sprite contact
					JSL $01802A             ; update position based on speed values

					LDA $14F8,x
					BNE NOT_MODE1
					JSR MODE1
					BRA MAIN_E
NOT_MODE1			CMP #$01
					BNE NOT_MODE2
					JSR MODE2
					BRA MAIN_E
NOT_MODE2			CMP #$02
					BNE NOT_MODE3
					JSR MODE3
					BRA MAIN_E
NOT_MODE3			CMP #$03
					BNE NOT_MODE4
					JSR MODE4
					BRA MAIN_E
NOT_MODE4			CMP #$04
					BNE NOT_MODE5
					JSR MODE5
					BRA MAIN_E
NOT_MODE5			CMP #$05
					BNE NOT_MODE6
					JSR MODE6
					BRA MAIN_E
NOT_MODE6           CMP #$06
					BNE MAIN_E
					JSR MODE7
MAIN_E				RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $94,$B4,$D4,$96,$B6,$D6,$88,$A2,$B0,$D0,$A8,$C8,$E8
                    dcb $94,$B4,$D4,$96,$B6,$D6,$88,$8C,$8C,$8C,$A8,$C8,$E8
                    dcb $94,$B4,$D4,$96,$B6,$D6,$88,$8C,$8C,$8C,$8C,$8C,$8C
HORIZ_DISP          dcb $F8,$F8,$F8,$08,$08,$08,$00,$E8,$D8,$D8,$18,$18,$18
VERT_DISP           dcb $F0,$00,$10,$F0,$00,$10,$E0,$F8,$00,$10,$F8,$08,$18

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder                 
                    LDA $1602,x
                    STA $03
                    PHX                     ; push sprite index

                    LDX #$0C                ; loop counter = (number of tiles per frame) - 1
LOOP_START          PHX                     ; push current tile number
                    
                    LDA $00                 ; \ tile x position = sprite x location ($00)
                    CLC                     ;  |
                    ADC HORIZ_DISP,x         ;  |
                    STA $0300,y             ; /   set x position of the tile
                    
                    LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /   set y position of the tile
                    
                    PHX
                    TXA
                    CLC
                    ADC $03
                    TAX
                    LDA TILEMAP,x           ; \ store tile
                    STA $0302,y             ; / 

                    LDA #$00
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | put in level properties
                    STA $0303,y             ; / store tile properties
                    PLX
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

                    PLX                     ; pull, X = sprite index
                    LDY #$02                ; \ 02, because we didn't write to 460 yet   460 = Tile Size 02:16x16
                    LDA #$0C                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ETC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPEED_X				dcb $20,$E0
MODE1				JSR DAMAGE_CHECK
					PHX
					LDA $157C,x
					TAX
					LDA SPEED_X,x
					PLX
					STA $B6,x
					STZ $AA,x
					LDA $E4,x
					CMP #$F0
					BCC MODE1_2
					LDA #$01
					STA $157C,x
					BRA MODE1_3
MODE1_2				CMP #$08
					BCS MODE1_3
					STZ $157C,x
MODE1_3				LDA $15AC,x
					BEQ MODE1_4
					RTS
MODE1_4				LDA $94
					AND #$F0
					STA $03
					LDA $E4,x
					AND #$F0
					CMP $03
					BNE MODE1_E
					LDA $13
					CLC
					ADC $94
					AND #$01
					BEQ MODE1_5
					LDA #$30
					STA $15AC,x
					LDA #$01
					STA $14F8,x
					BRA MODE1_E
MODE1_5				LDA #$60
					STA $15AC,x
					LDA #$03
					STA $14F8,x
MODE1_E				RTS

MODE2				JSR DAMAGE_CHECK
					STZ $B6,x
					STZ $AA,x
					LDA $15AC,x
					CMP #$01
					BNE MODE2_2
					LDA #$2B
					STA $1DFC
					BRA MODE2_E
MODE2_2				CMP #$00
					BNE MODE2_E
					LDA #$30
					STA $AA,x
					LDA $1588,x
					AND #$04
					BEQ MODE2_E
					LDA #$09
                    STA $1DFC
					LDA #$10
					STA $1887
					LDA #$30
					STA $15AC,x
					LDA #$02
					STA $14F8,x
MODE2_E				RTS

MODE3				JSR DAMAGE_CHECK
					STZ $B6,x
					STZ $AA,x
					LDA $15AC,x
					BNE MODE3_E
					LDA #$D0
					STA $AA,x
					LDA $D8,x
					CMP #$04
					BCS MODE3_E
					STZ $14F8,x
					LDA #$E3
					STA $15AC,x
MODE3_E				RTS


MODE4				JSR DAMAGE_CHECK
					STZ $B6,x
					STZ $AA,x
					LDA $15AC,x
					CMP #$40
					BNE MODE4_2
					LDA #$A2
					STA $1570,x
					JSR SPRITE_GEN
					LDA #$D0
                    STA $00AA,y
					LDA #$10
					STA $00B6,y
					JSR SPRITE_GEN
					LDA #$D0
                    STA $00AA,y
					LDA #$F0
					STA $00B6,y
					BRA MODE4_E
MODE4_2				CMP #$00
					BNE MODE4_E
					STZ $14F8,x
					LDA #$F7
					STA $15AC,x
MODE4_E				RTS

MODE5				STZ $AA,x
					STZ $B6,x
					LDA $15AC,x
					BNE MODE5_E
					LDA $7FFF05
					STA $14F8,x
					LDA $7FFF06
					STA $15AC,x
MODE5_E				RTS

MODE6				STZ $AA,x
					STZ $B6,x
					LDA $15AC,x
					BNE MODE6_2
					LDA $7FFF05
					STA $14F8,x
					LDA $7FFF06
					STA $15AC,x
					LDA $1602,x
					CLC
					ADC #$0D
					STA $1602,x
					RTS
MODE6_2				AND #$1F
					BNE MODE6_E
					LDA #$01
					STA $1570,x
					JSR SPRITE_GEN2
					INC $1594,x
					LDA #$10
					STA $15AC,y
					LDA #$10
					STA $1DFC
MODE6_E				RTS

MODE7				STZ $AA,x
					STZ $B6,x
					LDA $15AC,x
					BNE MODE7_2
					LDA #$02
					STA $1570,x
					JSR SPRITE_GEN2
					JSR DELETE_ALL
					LDA #$00
					STA $14F8,y
					STA $1528,y
					STZ $14C8,x
					RTS
MODE7_2				AND #$0F
					BNE MODE7_E
					LDA #$01
					STA $1570,x
					JSR SPRITE_GEN2
					INC $1594,x
					LDA #$10
					STA $15AC,y
					LDA #$10
					STA $1DFC
MODE7_E				RTS

DAMAGE_CHECK        LDA $E4,x
                    STA $7FFE01
                    LDA $14E0,x
                    STA $7FFE02
                    LDA $D8,x
                    STA $7FFE03
                    LDA $14D4,x
                    STA $7FFE04
                    PHX
                    LDX #$09
LOOP1               LDA $9E,x
                    CMP #$A2
                    BNE LOOP3
					LDA $14C8,x
					CMP #$09
					BNE LOOP3
					LDA $1588,x
					AND #$04
					BNE LOOP3
                    JSR NEAR_CHECK
                    TYA
                    BEQ LOOP3
                    LDA #$28
                    STA $1DFC
					LDA #$02
					STA $14C8,x
					TXY
					PLX
					INC $1528,x
					LDA $14F8,x
					STA $7FFF05
					LDA $15AC,x
					STA $7FFF06
					LDA $1528,x
					CMP #$02
					BEQ LOOP1_2
					CMP #$04
					BNE LOOP1_3
LOOP1_2				LDA #$05
					STA $14F8,x
					LDA #$7F
					STA $15AC,x
					BRA LOOP2
LOOP1_3				CMP #$06
					BNE LOOP1_4
					LDA #$6F
					STA $15AC,x
					LDA #$06
					STA $14F8,x
					BRA LOOP2
LOOP1_4				LDA #$04
					STA $14F8,x
					LDA #$20
					STA $15AC,x
LOOP2				PHX
					TYX
LOOP3               DEX
                    BPL LOOP1
                    PLX
                    RTS

DELETE_ALL          PHX
                    LDX #$09
LOOP21              LDA $9E,x
                    CMP #$A2
                    BNE LOOP22
                    STZ $14C8,x
LOOP22              DEX
                    BPL LOOP21
                    PLX
                    RTS



NEAR_CHECK          LDY #$00
                    LDA $E4,x
                    SEC
                    SBC $7FFE01
                    STA $0F
                    LDA $14E0,x
                    SBC $7FFE02
                    CMP #$00
                    BEQ RIGHT_CHECK
                    CMP #$FF
                    BNE CHECK_E
                    LDA $0F
                    CMP #$E8
                    BCC CHECK_E
                    BRA VER_CHECK
RIGHT_CHECK         LDA $0F
                    CMP #$18
                    BCS CHECK_E
VER_CHECK           LDA $D8,x
                    SEC
                    SBC $7FFE03
                    STA $0F
                    LDA $14D4,x
                    SBC $7FFE04
                    CMP #$00
                    BEQ UNDER_CHECK
                    CMP #$FF
                    BNE CHECK_E
                    LDA $0F
                    CMP #$E8
                    BCC CHECK_E
                    BRA NEAR
UNDER_CHECK         LDA $0F
                    CMP #$18
                    BCS CHECK_E
NEAR                LDY #$01
CHECK_E             RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite generator routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN67            RTS
SPRITE_GEN          JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURN67            ; / after: Y has index of sprite being generated
                    LDA #$08
                    STA $1DFC
                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /
                    LDA $1570,x
                    STA $009E,y             ; /set sprite number
                    LDA $E4,x               ; \ set x position for new sprite
                    STA $00E4,y

                    LDA $14E0,x             ;  |
                    STA $14E0,y             ; /

                    LDA $D8,x               ; \ set y position for new sprite
                    SEC
                    SBC #$20
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    SBC #$00
                    STA $14D4,y             ; /
                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    PLX                     ; / 
                    RTS                          ; return



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite generator routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_OFFSET            dcb $E0,$D8,$E8,$20,$28,$18,$E8,$10,$10,$F0,$18,$F0,$F8
X_OFFSET2           dcb $FF,$FF,$FF,$00,$00,$00,$FF,$00,$00,$FF,$00,$FF,$FF
Y_OFFSET            dcb $10,$00,$08,$10,$00,$08,$00,$F0,$10,$10,$00,$F0,$00
Y_OFFSET2           dcb $00,$00,$00,$00,$00,$00,$00,$FF,$00,$00,$00,$FF,$00

RETURN672           RTS
SPRITE_GEN2         JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURN672            ; / after: Y has index of sprite being generated
                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /
                    PHX
                    LDA NEW_SPRITE_NUM,x
                    CLC
                    ADC $1570,x
                    TYX
                    STA NEW_SPRITE_NUM,x
                    PLX
                    PHX
                    LDA $1594,x
                    TAX
                    LDA X_OFFSET,x
                    STA $02
                    LDA X_OFFSET2,x
                    STA $18D4
                    LDA Y_OFFSET,x
                    STA $03
                    LDA Y_OFFSET2,x
                    STA $0F
                    PLX
                    
                    LDA $E4,x               ;\ set x position for new sprite
                    CLC
                    ADC $02
                    STA $00E4,y

                    LDA $14E0,x             ;  |
                    ADC $18D4
                    STA $14E0,y             ; /

                    LDA $D8,x               ;\ set y position for new sprite
                    CLC
                    ADC $03
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    ADC $0F
                    STA $14D4,y             ; /
                    PHX                     ; \ before: X must have index of sprite being generated
                    TYX                     ;  | routine clears *all* old sprite values...
                    JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
                    JSL $0187A7             ;  get table values for custom sprite
                    LDA #$88
                    STA EXTRA_BITS,x
                    PLX                     ; / 
                    RTS                          ; return
