;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Vacume
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          JSR SUB_GFX             ; graphics routine
                    LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /
					LDA #$00
					STA $7FFA01
                    JSR DAMAGE_CHECK
					DEC $151C,x
					LDA $151C,x
					BNE MAIN2
					LDA #$02
					STA $7FFA01
					STZ $14C8,x
					RTS
MAIN2				JSR MODE1
					INC $1570,x
					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TILEMAP             dcb $0E,$2E,$2C,$2A
PROPERTIES          dcb $40,$00
X_OFFSET			dcb $08,$F0



SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder 
					LDA $1570,x
					LSR A
					LSR A
					AND #$03
					STA $03
					LDA $157C,x
					STA $02
                    PHX                     ; push sprite index

                                        
                    LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    STA $0301,y             ; /

					LDX $03
					LDA TILEMAP,x
                    STA $0302,y             ; / 

                    LDX $02
					LDA X_OFFSET,x
					CLC
					ADC $00                 ; \ tile x position = sprite x location ($00)
                    STA $0300,y             ; /                    
					
					LDA PROPERTIES,x
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | ?? what is in 64, level properties... disable layer priority??
                    STA $0303,y             ; / store tile properties

                    PLX                     ; pull, X = sprite index
                    
                    LDY #$02
                    LDA #$00                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MODE1				JSR SUB_GET_DIR
					CPY #$00
					BNE FACE_LEFT
					DEC $94
					BRA MODE1_2
FACE_LEFT			INC $94
MODE1_2				LDA $15AC,x
					BNE MODE1_E
					LDA #$20
					STA $15AC,x
					LDA #$17
					STA $1DFC
MODE1_E				RTS
					

					
DAMAGE_CHECK        LDA $E4,x
                    STA $7FFE01
                    LDA $14E0,x
                    STA $7FFE02
                    LDA $D8,x
                    STA $7FFE03
                    LDA $14D4,x
                    STA $7FFE04
                    PHX
                    LDX #$09
LOOP1               LDA $9E,x
                    CMP #$04
                    BNE LOOP3
					LDA $14C8,x
					CMP #$0A
					BNE LOOP3
                    JSR NEAR_CHECK
                    TYA
                    BEQ LOOP3
					
                    LDA #$03
                    STA $1DF9
					LDA #$01
					STA $7FFA01
					JSR DELETE_ALL
LOOP3               DEX
                    BPL LOOP1
                    PLX
                    RTS
					
NEAR_CHECK          LDY #$00
                    LDA $E4,x
                    SEC
                    SBC $7FFE01
                    STA $0F
                    LDA $14E0,x
                    SBC $7FFE02
                    CMP #$00
                    BEQ RIGHT_CHECK
                    CMP #$FF
                    BNE CHECK_E
                    LDA $0F
                    CMP #$F8
                    BCC CHECK_E
                    BRA VER_CHECK
RIGHT_CHECK         LDA $0F
                    CMP #$08
                    BCS CHECK_E
VER_CHECK           LDA $D8,x
                    SEC
                    SBC $7FFE03
                    STA $0F
                    LDA $14D4,x
                    SBC $7FFE04
                    CMP #$00
                    BEQ UNDER_CHECK
                    CMP #$FF
                    BNE CHECK_E
                    LDA $0F
                    CMP #$E8
                    BCC CHECK_E
                    BRA NEAR
UNDER_CHECK         LDA $0F
                    CMP #$28
                    BCS CHECK_E
NEAR                LDY #$01
CHECK_E             RTS


DELETE_ALL          PHX
                    LDX #$09
LOOP21              LDA $9E,x
                    CMP #$28
                    BEQ LOOP22
                    STZ $14C8,x
LOOP22              DEX
                    BPL LOOP21
                    PLX
                    RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURNK68           RTS     
GEN_SHELL           JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURNK68            ; / after: Y has index of sprite being generated

                    LDA #$09                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /

                    LDA #$04
                    STA $009E,y
                    
                    LDA $E4,x               ;\ set x position for new sprite
                    STA $00E4,y

                    LDA $14E0,x             ;  |
                    STA $14E0,y             ; /

                    LDA $D8,x               ;\ set y position for new sprite
					CLC
					ADC #$08
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
					ADC #$00
                    STA $14D4,y             ; /

                    PHX
                    TYX
                    JSL $07F7D2             ;  
                    PLX
                    RTS                     ; return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817                ; Y = 1 if contact

SUB_GET_DIR         LDY #$00                
                    LDA $94                 
                    SEC                     
                    SBC $E4,x               
                    STA $0F                 
                    LDA $95               
                    SBC $14E0,x            
                    BPL LABEL16             
                    INY                     
LABEL16             RTS                    

