;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Wall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    EXTRA_BITS = $7FAB10
                    NEW_SPRITE_NUM = $7FAB9E    ;08 bytes   custom sprite number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
                    dcb "INIT"
					STZ $C2,x
					LDA $E4,x
					CLC
					ADC #$08
					STA $E4,x
					LDA #$C0
					STA $15AC,x
					STZ $1570,x
					STZ $1528,x
					STZ $14F8,x
                    RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "MAIN"
                    PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
                    JSR CODE_START          ;  |
                    PLB                     ;  |
                    RTL                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN              RTS                     ; return           
CODE_START          LDA $C2,x
					BEQ MAIN
					JSR SUB_GFX             ; graphics routine
MAIN				LDA $14C8,x             ; \ 
                    CMP #$08                ;  | if status != 8, return
                    BNE RETURN              ; /
                    LDA $9D                 ; \ if sprites locked, return
                    BNE RETURN              ; /
					JSL $01802A             ; update position based on speed values
					LDA $16
					BEQ MAIN1
					LDA $18
					BEQ MAIN1
					INC $151C,x
MAIN1				JSR MOVE_EYE
					JSR MOVE_EYE2
					LDA $C2,x
					BNE NOT_MODE0
					JSR MODE0
					RTS
NOT_MODE0			CMP #$01
					BNE NOT_MODE1
					JSR MODE1
					RTS
NOT_MODE1			CMP #$02
					JSR MODE2
					RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $80,$A0
VERT_DISP           dcb $F8,$08

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder                
					PHX                     ; push sprite index
                    LDX #$01                ; loop counter = (number of tiles per frame) - 1
LOOP_START          PHX                     ; push current tile number
                    
                    
                    LDA $00                 ; \ tile y position = sprite y location ($01) + tile displacement
                    STA $0300,y             ; /   set y position of the tile
					
					LDA $01                ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /   set y position of the tile
                    
					LDA TILEMAP,x           ; \ store tile
                    STA $0302,y            ; / 
					
					LDA #$00            ;  |
                    ORA #$0F
					ORA $64
                    STA $0303,y             ; / store tile properties
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START          ; / 

                    PLX                     ; pull, X = sprite index
                    LDY #$02                ; \ 02, because we didn't write to 460 yet   460 = Tile Size 02:16x16
                    LDA #$01                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; routines below can be shared by all sprites.  they are ripped from original
; SMW and poorly documented
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ETC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RETURN0				RTS
MODE0				LDA $15AC,x
					BNE RETURN0
MODE00				LDA $1570,x
					CMP #$04
					BCC MODE01
					LDA #$01
					STA $C2,x
					STZ $14F8,x
					LDA #$FF
					STA $1887
					RTS
MODE01				LDA #$10
					STA $1DF9
					LDA $14D4,x
					PHA
					LDA $D8,x
					PHA
					LDA $14E0,x
					PHA
					LDA $E4,x
					PHA
					LDA #$01
					STA $14D4,x
					LDA #$44
					STA $D8,x
					STZ $14E0,x
					LDA #$54
					STA $E4,x
					JSR SUB_GET_X_DIFF
					JSR SUB_GET_Y_DIFF
					JSR GEN_SPRITE
					LDA #$9C
					STA $E4,x
					JSR SUB_GET_X_DIFF
					JSR SUB_GET_Y_DIFF
					JSR GEN_SPRITE
					PLA
					STA $E4,x
					PLA
					STA $14E0,x
					PLA
					STA $D8,x
					PLA
					STA $14D4,x
					LDA #$C0
					STA $15AC,x
					INC $1570,x
					RTS

MODE1				LDA $14F8,x
					BNE MODE10
					JSR CHECK_DAMAGE
MODE10				LDA $15AC,x
					BNE MODE12
					LDA $1887
					BEQ MODE11
					RTS
MODE11				LDA #$F0
					STA $15AC,x
					RTS
MODE12				CMP #$E0
					BNE MODE13
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$60
					STA $00B6,y
					LDA #$00
					STA $00AA,y
					RTS
MODE13				CMP #$D0
					BNE MODE14
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$40
					STA $00B6,y
					LDA #$20
					STA $00AA,y
					RTS
MODE14				CMP #$C0
					BNE MODE15
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$20
					STA $00B6,y
					LDA #$40
					STA $00AA,y
					RTS
MODE15				CMP #$B0
					BNE MODE17
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$00
					STA $00B6,y
					LDA #$60
					STA $00AA,y
					RTS
MODE17				CMP #$A0
					BNE MODE18
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$E0
					STA $00B6,y
					LDA #$40
					STA $00AA,y
					RTS
MODE18				CMP #$90
					BNE MODE19
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$C0
					STA $00B6,y
					LDA #$20
					STA $00AA,y
					RTS
MODE19				CMP #$80
					BNE MODE1A
					LDA #$09
					STA $1DFC
					JSR GEN_SPRITE
					LDA #$A0
					STA $00B6,y
					LDA #$00
					STA $00AA,y
					RTS
MODE1A				CMP #$70
					BNE RETURN1A
					LDA #$80
					STA $15AC,x
					STZ $C2,x
					STZ $1570,x
					LDA $1528,x
					CMP #$03
					BCC RETURN1A
					LDA #$02
					STA $C2,x
RETURN1A			RTS

MODE2				JSR CHECK_DAMAGE
					LDA #$20
					STA $1570,x
					LDA #$10
					STA $151C,x
					LDA $94
					STA $7FFA00
					LDA $95
					STA $7FFA01
					LDA $96
					STA $7FFA02
					LDA $97
					STA $7FFA03
					JSR SPEED_DEC
					LDA $1594,x
                    STA $AA,x
                    LDA $1504,x
                    STA $B6,x
MODE2_0				LDA $1594,x
					CMP #$80
					BCS MODE2_4
					CMP #$18
					BCC MODE2_E
					LSR A
					STA $AA,x
MODE2_3_1			LDA $B6,x
					CMP #$80
					BCS MODE2_3_2
					LSR A
					STA $B6,x
					RTS
MODE2_3_2			LDA #$00
					SEC
					SBC $B6,x
					LSR A
					STA $B6,x
					LDA #$00
					SEC
					SBC $B6,x
					STA $B6,x
MODE2_E				RTS
MODE2_4				CMP #$F8
					BCS MODE2_E
					LDA #$F8
					STA $AA,x
					BRA MODE2_3_1
					RTS

MOVE_EYE			STZ $AA,x
					STZ $B6,x
					STZ $1467
					LDA $94
					CMP #$20
					BCC MOVE_LEFT
					CMP #$30
					BCC MOVE_LEFT1
					CMP #$40
					BCC MOVE_LEFT2
					CMP #$50
					BCC MOVE_LEFT3
					CMP #$60
					BCC MOVE_LEFT4
					CMP #$70
					BCC MOVE_LEFT5
					CMP #$D0
					BCS MOVE_RIGHT
					CMP #$C0
					BCS MOVE_RIGHT1
					CMP #$B0
					BCS MOVE_RIGHT2
					CMP #$A0
					BCS MOVE_RIGHT3
					CMP #$90
					BCS MOVE_RIGHT4
					CMP #$80
					BCS MOVE_RIGHT5
					STZ $1466
					RTS
MOVE_LEFT			LDA #$06
					STA $1466
					RTS
MOVE_LEFT1			LDA #$05
					STA $1466
					RTS
MOVE_LEFT2			LDA #$04
					STA $1466
					RTS
MOVE_LEFT3			LDA #$03
					STA $1466
					RTS
MOVE_LEFT4			LDA #$02
					STA $1466
					RTS
MOVE_LEFT5			LDA #$01
					STA $1466
					RTS
					
MOVE_RIGHT			LDA #$FA
					STA $1466
					LDA #$FF
					STA $1467
					RTS
MOVE_RIGHT1			LDA #$FB
					STA $1466
					LDA #$FF
					STA $1467
					RTS
MOVE_RIGHT2			LDA #$FC
					STA $1466
					LDA #$FF
					STA $1467
					RTS
MOVE_RIGHT3			LDA #$FD
					STA $1466
					LDA #$FF
					STA $1467
					RTS
MOVE_RIGHT4			LDA #$FE
					STA $1466
					LDA #$FF
					STA $1467
					RTS
MOVE_RIGHT5			LDA #$FF
					STA $1466
					LDA #$FF
					STA $1467
					RTS

MOVE_EYE2			LDA $97
					BEQ MOVE_UP1
					LDA $96
					CMP #$10
					BCC MOVE_UP2
					CMP #$20
					BCC MOVE_UP3
					CMP #$30
					BCC MOVE_UP4
					LDA #$C0
					STA $1468
					RTS
MOVE_UP1			LDA #$C4
					STA $1468
					RTS
MOVE_UP2			LDA #$C3
					STA $1468
					RTS
MOVE_UP3			LDA #$C2
					STA $1468
					RTS
MOVE_UP4			LDA #$C1
					STA $1468
					RTS

CHECK_DAMAGE		JSL $01A7DC
					BCC NO_DAMAGE
					LDA $7D
					CMP #$10
					BMI ENE_WINS
					INC $1528,x
					LDA $1528,x
					CMP #$04
					BEQ THEEND
					INC $14F8,x
					JSL $01AA33             ; set mario speed
					JSL $01AB99             ; display contact graphic
					LDA #$20
                    STA $1DF9
NO_DAMAGE			RTS

ENE_WINS            LDA $1497               ; \ if mario is invincible...
                    BNE RETURN2             ; /   ... return
                    JSL $00F5B7             ; hurt mario
RETURN2             RTS
THEEND				LDA #$0B
                    STA $1DFB
					STZ $40
                    LDA #$FF
                    STA $13C6
                    STA $1493
					LDA #$08
                    STA $1DF9
					STZ $14C8,x
					JSR SUB_SMOKE2
					RTS
SUB_SMOKE2          LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open
FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $01
                    STA $17C4,y             ; /
                    LDA #$1B                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $00
                    STA $17C8,y             ; /
                    RTS   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; generate sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Y_MAGIC             dcb $04,$04,$02,$02,$FC,$FC,$FE,$FE
X_MAGIC             dcb $0C,$F4,$10,$F0

RETURNK68           RTS     
GEN_SPRITE          JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
                    BMI RETURNK68            ; / after: Y has index of sprite being generated

                    LDA #$08                ; \ set sprite status for new sprite
                    STA $14C8,y             ; /

                    LDA #$20
                    STA $009E,y
                    
                    LDA $E4,x               ;\ set x position for new sprite
                    STA $00E4,y

                    LDA $14E0,x             ;  |
                    STA $14E0,y             ; /

                    LDA $D8,x               ;\ set y position for new sprite
					STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
					STA $14D4,y             ; /

                    PHX
                    TYX
                    JSL $07F7D2             ;  
                    PLX

                    PHX                     ; \ set y speed of fire ball
                    LDA $1602,x             ;  |
                    ASL A                   ;  |
                    ASL A                   ;  |
                    ORA $157C,x             ;  |
                    TAX                     ;  |
                    LDA Y_MAGIC,x            ;  |
                    STA $00AA,y             ;  |
                    PLX                     ; /     
                    
                    PHX                     ; \  set x speed of fire ball
                    LDA $157C,x             ;  |
                    TAX                     ;  |
                    LDA X_MAGIC,x            ;  |
                    STA $00B6,y             ; /
                    PLX
                    RTS                     ; return
					
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; difference in y position
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GET_Y_DIFF      LDY #$00                
                    LDA $D3                 
                    PHA
                    CLC                     
                    ADC #$10                
                    STA $D3                 
                    LDA $D4                 
                    PHA
                    ADC #$00                
                    STA $D4                 

                    LDA $D3                 
                    SEC                     
                    SBC $D8,x               
                    STA $0E                 
                    LDA $D4                 
                    SBC $14D4,x             
                    BPL LABEL14            
                    INY                     
LABEL14             TYA                     ;  | 
                    STA $1602,x 
                    
                    PLA
                    STA $D4
                    PLA
                    STA $D3
                    
                    RTS                    

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; difference in x position
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   

SUB_GET_X_DIFF      LDY #$00                
                    LDA $D1                 
                    SEC                    
                    SBC $E4,x              
                    STA $0F                 
                    LDA $D2                 
                    SBC $14E0,x            
                    BPL TO_RIGHT            
TO_LEFT             INY                     
                    LDA $0F
                    CMP #$B0
                    BCS SET_VAL
                    INY
                    INY                 
                    BRA SET_VAL
TO_RIGHT            LDA $0F
                    CMP #$50
                    BCC SET_VAL
                    INY
                    INY 
SET_VAL             TYA
                    STA $157C,x 
                    RTS                     

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPEED_DEC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   

SPEED_DEC           LDA $E4,x
                    SEC
                    SBC $7FFA00
                    STA $0F
                    LDA $14E0,x
                    SBC $7FFA01
                    BPL FACE_LEFT
                    JSR RIGHT_SPEED
                    BRA VERC
FACE_LEFT           CMP #$01
                    BEQ LEFT2
                    JSR LEFT_SPEED1
                    BRA VERC
LEFT2               JSR LEFT_SPEED2
VERC                LDA $D8,x
                    SEC
                    SBC $7FFA02
                    STA $0F
                    LDA $14D4,x
                    SBC $7FFA03
                    BPL FACE_UP
                    JSR DOWN_SPEED
                    LDA $1594,x
                    CMP #$10
                    BCS VER_E
                    LDA #$10
                    STA $1594,x
                    BRA VER_E
FACE_UP             JSR UP_SPEED
                    LDA $1594,x
                    CMP #$F0
                    BCC VER_E
                    LDA #$F0
                    STA $1594,x
VER_E               RTS

UP_SPEED            LDA $0F38
                    CMP #$00
                    BNE UP3
                    LDA $0F
                    CMP #$80
                    BCC UP2
                    LDA #$7F
UP2                 STA $0F
                    BRA UP5
UP3                 CMP #$02
                    BNE UP4
                    JSR UP1_1
                    BRA UP5
UP4                 JSR UP1_2
UP5                 LDA #$00
                    SEC
                    SBC $0F
                    STA $1594,x
                    RTS

DOWN_SPEED          LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    LDA $0F38
                    CMP #$00
                    BNE DOWN3
                    LDA $0F
                    CMP #$80
                    BCC DOWN2
                    LDA #$7F
DOWN2               STA $1594,x
                    RTS
DOWN3               CMP #$02
                    BNE DOWN4
                    JSR UP1_1
                    BRA DOWN5
DOWN4               JSR UP1_2
DOWN5               LDA $0F
                    STA $1594,x
                    RTS
UP1_1
LOOPUP1             LDA $0F37
                    BEQ LOOPUP1E
                    LDA $0F
                    LSR A
                    STA $0F
                    DEC $0F37
                    LDA $0F37
                    BNE LOOPUP1
LOOPUP1E            LDA $0F
                    CMP #$80
                    BCC LOOPUP1EE
                    LDA #$7F
                    STA $0F
LOOPUP1EE           RTS

UP1_2               LDA $0F
                    CMP #$80
                    BCC LOOPUP3
                    LDA #$7F
                    STA $0F
                    STZ $0F37
                    RTS
LOOPUP3             LDA $0F37
                    BEQ LOOPUP2E
                    LDA $0F
                    ASL A
                    STA $0F
                    DEC $0F37
                    LDA $0F37
                    BNE UP1_2
LOOPUP2E            LDA $0F
                    CMP #$80
                    BCC LOOPUP2EE
                    LDA #$7F
                    STA $0F
LOOPUP2EE           RTS







LEFT_SPEED1         STZ $0F38
                    LDA $0F
                    CMP $1570,x
                    BCC LEFT1_2
                    LDA #$02
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_40
                    BRA LEFT1_3
LEFT1_2             CMP $151C,x
                    BCS LEFT1_3
                    LDA #$01
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_20
                    BRA LEFT1_3
LEFT1_3             LDA #$00
                    SEC
                    SBC $0F
                    STA $1504,x
                    RTS

LEFT1_40            
LOOP140             LDA $0F
                    CMP $1570,x
                    BCC LOOP140E
                    LSR A
                    STA $0F
                    INC $0F37
                    BRA LOOP140
LOOP140E            RTS
LEFT1_20            
LOOP120             LDA $0F
                    CMP $151C,x
                    BCS LOOP120E
                    ASL A
                    STA $0F
                    INC $0F37
                    BRA LOOP140
LOOP120E            RTS

LEFT_SPEED2         STZ $0F37
                    INC $0F37
                    LDA $0F
                    LSR A
                    CLC
                    ADC #$80
                    STA $0F
                    LDA #$02
                    STA $0F38
                    JSR LEFT1_40
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $1504,x
                    RTS

RIGHT_SPEED         STZ $0F38
                    LDA #$00
                    SEC
                    SBC $0F
                    STA $0F
                    CMP $1570,x
                    BCC RIGHT_2
                    LDA #$02
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_40
                    BRA RIGHT_3
RIGHT_2             CMP $151C,x
                    BCS RIGHT_3
                    LDA #$01
                    STA $0F38
                    STZ $0F37
                    JSR LEFT1_20
                    BRA RIGHT_3
RIGHT_3             LDA $0F
                    STA $1504,x
                    RTS

SUB_GET_DIR         LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31

